%{ open Ast %}


%token SEMICOLON LEFT_PAREN RIGHT_PAREN BEGIN_STATEMENT END_STATEMENT LEFT_SQ_BRACKET RIGHT_SQ_BRACKET COMMA

%token MATRIX_POWER DECIMAL_POINT
%token PLUS MINUS TIMES DIVIDE ASSIGNMENT IS
%token EQUAL_TO NOT_EQUAL_TO LESS_THAN LESS_THAN_EQUAL_TO GREATER_THAN GREATER_THAN_EQUAL_TO

%token RETURN_STATEMENT IF_CONDITION ELSE_CONDITION FOR WHILE INT
 FLOATLIT VARIABLE_ASSIGNMENT
%token <int> LITERAL

%token <string> ID
%token <float> FLOAT
%token MINIMUM MAXIMUM MEDIAN_PRICE
%token EMA_AVERAGE SMA_AVERAGE OPEN_PRICE CLOSE_PRICE HIGH_PRICE LOW_PRICE
%token ERROR_TRY ERROR_CATCH ERROR_THROW ERROR_NUM
%token BEGIN_LOOP END_LOOP
%token EOF


%nonassoc NOELSE

%nonassoc NOTRY
%nonassoc ELSE_CONDITION


%right ASSIGNMENT

%left EQUAL_TO NOT_EQUAL_TO

%left LESS_THAN GREATER_THAN LESS_THAN_EQUAL_TO GREATER_THAN_EQUAL_TO

%left PLUS MINUS

%left TIMES DIVIDE



%start program

%type <Ast.program> program



%%



program:
   /* nothing */ { [], [] }
 
| program vdecl { ($2 :: fst $1), snd $1 }
 
| program fdecl { fst $1, ($2 :: snd $1) }



fdecl:
   ID LEFT_PAREN formals_opt RIGHT_PAREN BEGIN_STATEMENT vdecl_list stmt_list END_STATEMENT
     { { fname = $1;
	 formals = $3;
	 locals = List.rev $6;
	 body = List.rev $7 } }



formals_opt:
    /* nothing */ { [] }
  
| formal_list   { List.rev $1 }



formal_list:
    
  ID                   { [$1] }
  
| formal_list COMMA ID { $3 :: $1 }



vdecl_list:
    
  /* nothing */    { [] }
  
| vdecl_list vdecl { $2 :: $1 }



vdecl:
   
	  INT ID SEMICOLON { $2 }

	| FLOATLIT ID SEMICOLON { $2 }


	| MINIMUM ID SEMICOLON { $2 }
	| MAXIMUM ID SEMICOLON { $2 }
	| MEDIAN_PRICE ID SEMICOLON { $2 }
	| EMA_AVERAGE ID SEMICOLON { $2 }
	| SMA_AVERAGE ID SEMICOLON { $2 }
	| OPEN_PRICE ID SEMICOLON { $2 }
	| CLOSE_PRICE ID SEMICOLON { $2 }
	| HIGH_PRICE ID SEMICOLON { $2 }
	| LOW_PRICE ID SEMICOLON { $2 }
	| ERROR_NUM ID SEMICOLON { $2 }
	





stmt_list:
    /* nothing */  { [] }
  
| stmt_list stmt { $2 :: $1 }



stmt:
    
  expr SEMICOLON { Expr($1) }
  
| RETURN_STATEMENT expr SEMICOLON { Return($2) }
  
| BEGIN_STATEMENT stmt_list END_STATEMENT { Block(List.rev $2) }
  
| IF_CONDITION LEFT_PAREN expr RIGHT_PAREN stmt %prec NOELSE { If($3, $5, Block([])) }
  
| IF_CONDITION LEFT_PAREN expr RIGHT_PAREN stmt ELSE_CONDITION stmt    { If($3, $5, $7) }
  
| FOR LEFT_PAREN expr_opt SEMICOLON expr_opt SEMICOLON expr_opt RIGHT_PAREN stmt
     { For($3, $5, $7, $9) }
  
| WHILE LEFT_PAREN expr RIGHT_PAREN stmt { While($3, $5) }


| ERROR_TRY BEGIN_STATEMENT stmt_list END_STATEMENT { Block(List.rev $3) }
  
| ERROR_CATCH BEGIN_STATEMENT stmt_list END_STATEMENT { Block(List.rev $3) }
  
| ERROR_THROW expr SEMICOLON { Throw($2) }
  

expr_opt:
    /* nothing */ { Noexpr }
  
| expr          { $1 }



expr:
    
  LITERAL          			{ Literal($1) }
  
| ID               			{ Id($1) }
  
| expr PLUS   			expr    { Binop($1, Add,   $3) }
  
| expr MINUS  			expr    { Binop($1, Sub,   $3) }
  
| expr TIMES  			expr    { Binop($1, Mult,  $3) }
  
| expr DIVIDE 			expr    { Binop($1, Div,   $3) }
  
| expr EQUAL_TO			expr    { Binop($1, Equal, $3) }
  
| expr NOT_EQUAL_TO    		expr    { Binop($1, Neq,   $3) }
  
| expr LESS_THAN     		expr    { Binop($1, Less,  $3) }
  
| expr LESS_THAN_EQUAL_TO    	expr    { Binop($1, Leq,   $3) }
  
| expr GREATER_THAN     	expr    { Binop($1, Greater,  $3) }
  
| expr GREATER_THAN_EQUAL_TO    expr    { Binop($1, Geq,   $3) }
  
| ID ASSIGNMENT 		expr    { Assign($1, $3) }
  
| ID LEFT_PAREN actuals_opt RIGHT_PAREN { Call($1, $3) }
  
| LEFT_PAREN 			expr RIGHT_PAREN { $2 }





actuals_opt:
    /* nothing */ { [] }
  
| actuals_list  { List.rev $1 }




actuals_list:
    expr { [$1] }
  
| actuals_list COMMA expr { $3 :: $1 }
