{ open Parser }

rule token = parse
  [' ' '\t' '\r' '\n'] { token lexbuf } (* Whitespace *)

| "[*"     { comment lexbuf }           (* Comments *)

| '('      { LEFT_PAREN }

| ')'      { RIGHT_PAREN }

| "begin"  { BEGIN_STATEMENT }

| "end"    { END_STATEMENT }

| ';'      { SEMICOLON }

| '[' 	   { LEFT_SQ_BRACKET }
| ']' 	   { RIGHT_SQ_BRACKET }
| '^' 	   { MATRIX_POWER }
| '.' 	   { DECIMAL_POINT }
| ','      { COMMA }

| '+'      { PLUS }

| '-'      { MINUS }

| '*'      { TIMES }

| '/'      { DIVIDE }

| '='      { ASSIGNMENT }

| "=="     { EQUAL_TO }

| "!="     { NOT_EQUAL_TO }

| '<'      { LESS_THAN }

| "<="     { LESS_THAN_EQUAL_TO }

| ">"      { GREATER_THAN }

| ">="     { GREATER_THAN_EQUAL_TO }
| "var"    { VARIABLE_ASSIGNMENT }

| "if"     { IF_CONDITION }

| "else"   { ELSE_CONDITION }

| "for"    { FOR }

| "while"  { WHILE }

| "return" { RETURN_STATEMENT }

| "begin"  { BEGIN_LOOP }
| "end"    { END_LOOP }
| "try"    { ERROR_TRY }
| "catch"  { ERROR_CATCH }
| "throw"  { ERROR_THROW }
| "errno"  { ERROR_NUM }
| "int"    { INT }

| "float"  { FLOATLIT }
| ['0'-'9']+ ['.'] ['0'-'9']* ('e' ['-' '+']? ['0'-'9']+)? as lxm { FLOAT (float_of_string lxm) }
| "min"    { MINIMUM }
| "max"    { MAXIMUM }
| "median"  { MEDIAN_PRICE }
| "ema"    { EMA_AVERAGE }
| "sma"    { SMA_AVERAGE }
| "open"   { OPEN_PRICE }
| "close"  { CLOSE_PRICE }
| "high"   { HIGH_PRICE }
| "low"   { LOW_PRICE }
| ['0'-'9']+ as lxm { LITERAL(int_of_string lxm) }

| ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { ID(lxm) }

| eof { EOF }

| _ as char { raise (Failure("Error in command. Please refer TradersJoe language reference manual " ^ Char.escaped char)) }

 and comment = parse
  "*]" { token lexbuf }

| _    { comment lexbuf }

